#include <client.hpp>
#include <client_p.hpp>

namespace myx {

namespace redis {

ClientPrivate::ClientPrivate( Client* client ) :
	lexer ( &socket ),
	parser( &lexer )
{
	connect( &socket, &QTcpSocket::connected, client, &Client::connected );
	connect( &socket, &QTcpSocket::disconnected, client, &Client::disconnected );
	connect( &parser, &Parser::reply, this, &ClientPrivate::sendReply );
}


void ClientPrivate::sendReply( const Reply& reply )
{
	Q_EMIT queue.dequeue()->reply( reply );
}


Client::Client( QObject* parent ) :
	QObject( parent ),
	d      ( new ClientPrivate( this ) )
{
}


void Client::connectToHost( const QString& hostName, quint16 port )
{
	d->socket.connectToHost( hostName, port );
}


void Client::disconnectFromHost()
{
	d->socket.disconnectFromHost();
}


bool Client::isConnected() const
{
	return( d->socket.state() == QAbstractSocket::ConnectedState );
}


Request* Client::sendCommand( const QByteArray& command )
{
	d->socket.write( command + "\r\n" );

	auto* request = new Request( this );
	d->queue.enqueue( request );
	return( request );
}


bool Client::waitForConnected( int msecs )
{
	return( d->socket.waitForConnected( msecs ) );
}


bool Client::waitForDisconnected( int msecs )
{
	return( d->socket.waitForDisconnected( msecs ) );
}

} // namespace redis

} // namespace myx
